% Programm, welches die Auswertung steuert
% AUTOR: Sebastian Behr


loadFiles :- ['..\\Python-Interface\\dyn_LEX'],
             ['..\\Grammatik\\g20'],
             ['..\\IO\\eingabe'].





%Entweder die Analyse des Satzes gelingt, oder wir analysieren den Fehler genauer
go :- loadFiles, eingabe(X), write('Eingabesatz: '), writeln(X), findall(r(ok,T), s([ok],T,X,[]), L), zeilen(L,1), ! ;
      eingabe(X), Err \== [ok], findall(Err, s(Err,T,X,[]), L), errorMessages(L), writeln('Lexikoneintrge:'), lexError(X), npKongruenz(X), partAnalyse(X).




%doppelte Ergebnisse, die durch lexikalische oder syntaktische Ambiguitten entstehen

%zeilenweise Ausgabe der Ergebnisse
zeilen([],1) :- fail.
zeilen([],N) :- N > 1.
zeilen([r(ok,T)|B],N) :- write('Lsung '), write(N), writeln(' :'),  writeln(T), M is N + 1, zeilen(B,M).



errorMessages(A) :- filterDuplicates(A,B), genericError(B).


%doppelte Fehlermeldungen entfernen
filterDuplicates([],[]).
filterDuplicates([A|B], [A|D]) :- dropFromRest(A,B,C), filterDuplicates(C,D).

%alle Elemente aus Restliste entfernen, die mit Vergleichselement identisch sind.
dropFromRest(_,[],[]).
dropFromRest(A,[B|C],[B|D]) :- A \== B, dropFromRest(A,C,D).
dropFromRest(A,[A|C],D) :- dropFromRest(A,C,D).



genericError(C) :- writeln('Der Eingabesatz ist ungrammatisch.'), errCodeToMessage(C).

errCodeToMessage([]).
errCodeToMessage([[numErr]              |B]) :- writeln('SUBJEKT <--> VERB : NUMERUS stimmt nicht berein.'), errCodeToMessage(B).
errCodeToMessage([[missingObj]          |B]) :- writeln('VERB : OBJEKT fehlt oder ist falsch.'), errCodeToMessage(B).
errCodeToMessage([[anapherKongruenzFail]|B]) :- writeln('Kein passendes BEZUGSWORT fr ANAPHER vorhanden.'), errCodeToMessage(B).


%%%%%%%%%
%Finden sich die Worte der Eingabe im Lexikon?
lexError([]).
lexError([A|B]) :- findall(L1,anyCategory(A,L1),L2), countOccurances(L2,0,Num), showLexEntry(A,Num), lexError(B).


%Findet alle Eintrge eines Wortes aus einer lexikalischen Kategorie
anyCategory(A,L) :- findall(MP,n0(MP,[A],[]),L).
anyCategory(A,L) :- findall(MP,a0(MP,[A],[]),L).
anyCategory(A,L) :- findall(_,adv0([A],[]),L).
anyCategory(A,L) :- findall(MP,v0(MP,_,[A],[]),L).
anyCategory(A,L) :- findall(MP,p0(MP,[A],[]),L).
anyCategory(A,L) :- findall(_,koord0([A],[]),L).
anyCategory(A,L) :- findall(_,part([A],[]),L).
anyCategory(A,L) :- findall(_,c0([A],[]),L).
anyCategory(A,L) :- findall(MP,det(MP,[A],[]),L).
anyCategory(A,L) :- findall(MP,ana(MP,_,[A],[]),L).
anyCategory(A,L) :- findall(MP,pron(MP,[A],[]),L).
anyCategory(A,L) :- findall(MP,pospron(MP,[A],[]),L).
anyCategory(A,L) :- findall(MP,wwort(MP,_,[A],[]),L).


countOccurances([],Num,Num).
countOccurances([A|B],Num,Max) :- length(A,Num_sub), Num >= Num_sub, countOccurances(B,Num,Max).
countOccurances([A|B],Num,Max) :- length(A,Num_sub), Num <  Num_sub, countOccurances(B,Num_sub,Max).


showLexEntry(WORT,0) :- write(WORT), writeln(' : Wort nicht im Lexikon'), !.
showLexEntry(WORT,Nr) :-write(WORT), write(' : '), write(Nr), writeln(' Eintrge im Lexikon').


%%%%%%%%%
%welche Nominalphrasen finden sich?

npKongruenz(X) :- writeln('Einzelne Nominalphrasen analysieren:'), bufferNPs(X,[],[],Res1), narrowNPs(Res1,[],Res2), showNPs(Res2).

%erfasst jeweils alle links von N befindlichen Worte
bufferNPs([],_,Buf1,Buf1).
bufferNPs([A|B],TBuf1,Buf1,Res) :- n0(_,[A],[]), append(TBuf1,[A],TBuf2), append(Buf1,[TBuf2],Buf2), bufferNPs(B,TBuf2,Buf2,Res), !.
bufferNPs([A|B],TBuf1,Buf,Res) :- append(TBuf1,[A],TBuf2), bufferNPs(B,TBuf2,Buf,Res).

%erhlt eine Liste von Listen. Gibt eine Liste von Bumen aus
%krzt die N0 vorausgehenden Worte so lange weg, bis sich eine NP bilden lsst
narrowNPs([],Res,Res).
narrowNPs([A|B],Pre1,Res) :- n2(M,1,_,_,_,T,A,[]), append(Pre1,[(A,M,T)],Pre2), narrowNPs(B,Pre2,Res), !.
%narrowNPs([[]|C],Pre,Res) :- narrowNPs(C,Pre,Res).
narrowNPs([[_|B]|C],Pre,Res) :- narrowNPs([B|C],Pre,Res).


showNPs([]).
showNPs([(A,B,C)|D]) :- write(A), writeln(' :'), showNP_MP(B), writeln(C), showNPs(D).

showNP_MP(mp(_,Numerus,Genus,Kasus,_,_,_,_,_,_)) :- write('Numerus: '), mapNumerus(Numerus), write(', Genus: '), mapGenus(Genus), write(', Kasus: '), mapKasus(Kasus), writeln('').




%%%%%%%%%
%analysiert, ob sich zumindest ein Nebensatz erfolgreich parsen lsst


partAnalyse(S) :- findeTeilsatz(S,keinerGefunden).


findeTeilsatz([],gefunden).
findeTeilsatz([],keinerGefunden) :- writeln('keine separate Nebensatz-Analyse mglich').
findeTeilsatz([A|B],_) :- c0([A],[]), analysiereTeilsatz([A|B]), findeTeilsatz(B,gefunden), !.
findeTeilsatz([_|B],Fund) :- findeTeilsatz(B,Fund).

analysiereTeilsatz(S) :- writeln('Versuche Teilsatz-Anaylse...'), findall((MP,T,S,REST), teil_s(MP,T,S,REST), L), showPartialTrees(L), !. 

%Analysiere den Nebensatz
%ACHTUNG : 8. KOMPONENTE VON C2 NOCH DEFINIEREN!!!
teil_s(none,T) --> c2(1,[empty,empty,empty,empty,empty],empty,SubkatUP,RectDom,T,cpbesetzung(_,_,nebensatz),[ok]), {dStrukturSchlingeLeer(SubkatUP,[ok]), allAlphasBound(RectDom)}.
teil_s(MP,T) --> c2(1,[empty,empty,empty,c(np,MP,nonIE,nonIE,selegiert,keinMiniBarrSchutz,0,0),empty],empty,SubkatUP,RectDom,T,cpbesetzung(_,_,nebensatz),[ok]), {dStrukturSchlingeLeer(SubkatUP,[ok]), allAlphasBound(RectDom)}.
teil_s(MP,T) --> c2(1,[empty,empty,c(np,MP,nonIE,nonIE,selegiert,keinMiniBarrSchutz,0,0),empty,empty],empty,SubkatUP,RectDom,T,cpbesetzung(_,_,nebensatz),[ok]), {dStrukturSchlingeLeer(SubkatUP,[ok]), allAlphasBound(RectDom)}.
teil_s(MP,T) --> c2(1,[empty,c(np,MP,nonIE,nonIE,selegiert,keinMiniBarrSchutz,0,0),empty,empty,empty],empty,SubkatUP,RectDom,T,cpbesetzung(_,_,nebensatz),[ok]), {dStrukturSchlingeLeer(SubkatUP,[ok]), allAlphasBound(RectDom)}.

showPartialTrees([]).
showPartialTrees([(none,T,S,REST)|B]) :- differenz(S,REST,DIFF_S), write('Teilsatz erfolgreich analysiert: '), writeln(DIFF_S), writeln(T), showPartialTrees(B).
showPartialTrees([(mp(_,_,_,Kasus,_,_,_,_,_,_,_),T)|B]) :- write('[ ... <'), mapKasus(Kasus), write('-Objekt>   ...  '), write(T), writeln(' ... ]'), showPartialTrees(B).


mapNumerus(sg) :- write('SINGULAR').
mapNumerus(pl) :- write('PLURAL').

mapGenus(mas) :- write('MASKULINUM').
mapGenus(fem) :- write('FEMININUM').
mapGenus(neu) :- write('NEUTRUM').

mapKasus(nom) :- write('NOMINATIV').
mapKasus(gen) :- write('GENITIV').
mapKasus(dat) :- write('DATIV').
mapKasus(akk) :- write('AKKUSATIV').

%%%%%%%%%%%%%%%%%%%
% Mengenoperationen
%%%%%%%%%%%%%%%%%%%

%Gibt die 1. Liste zurck ohne die Elemente der 2. Liste
differenz([],[],[]) :- !.
differenz(A,[],A) :- A \== [], !.
differenz([],B,[]) :- B \== [], !.
differenz([A1|R1],B,[A1|C]) :- not(member(A1,B)), differenz(R1,B,C), !.
differenz([_|R1],B,C) :- differenz(R1,B,C).


